# -*- coding: utf-8 -*-
import datetime
import icemac.ab.importer.interfaces
import os.path
import sys
import unittest
import zope.interface.verify


class ReaderTest(unittest.TestCase):
    """Test class for readers.

    Extend it and set the class variables to test your reader implementation.

    The import files have to be in a `test/data` directory relative to the
    importer class.
    """

    reader_class = None  # reference to the reader's class object
    import_file = None  # name of the longer import file
    import_file_short = None  # name of the shorter import file

    def getFileHandle(self, file_name=None):
        base_path = sys.modules[self.reader_class.__module__].__file__
        if file_name is None:
            file_name = self.import_file
        return file(os.path.join(
            os.path.dirname(base_path), 'tests', 'data', file_name))

    def getReader(self, import_file=None):
        return self.reader_class.open(self.getFileHandle(import_file))

    def test_Reader__interfaces__1(self):
        """It fulfills the IImportFileReader interface."""
        assert zope.interface.verify.verifyObject(
            icemac.ab.importer.interfaces.IImportFileReader,
            self.getReader())

    def test_Reader__getFieldNames__1(self):
        """It returns a list of unicode field names."""
        field_names = list(self.getReader().getFieldNames())
        assert [u'last name', u'firstname', u'birth_date'] == field_names
        assert isinstance(field_names[0], unicode)
        assert isinstance(field_names[1], unicode)
        assert isinstance(field_names[2], unicode)

    def test_Reader__getFieldSamples__1(self):
        """It returns a list of unicode field values."""
        samples = list(self.getReader().getFieldSamples(u'firstname'))
        assert [u'Andreas', u'Hanna', u'Jens'] == samples
        assert isinstance(samples[0], unicode)
        assert isinstance(samples[1], unicode)
        assert isinstance(samples[2], unicode)

    def test_Reader__getFieldSamples__2(self):
        """It is able to handle non-ASCII field values."""
        assert ([u'Koch', u'Hula', u'Jänsen'] ==
                list(self.getReader().getFieldSamples(u'last name')))

    def test_Reader__getFieldSamples__3(self):
        """It returns datetime field values as ISO unicodes."""
        samples = list(self.getReader().getFieldSamples(u'birth_date'))
        assert [u'1976-01-24', u'2000-01-01', u''] == samples
        assert isinstance(samples[0], unicode)
        assert isinstance(samples[1], unicode)
        assert isinstance(samples[2], unicode)

    def test_Reader__getFieldSamples__4(self):
        """It returns all samples if there are less than 3 in the file."""
        samples = list(self.getReader(self.import_file_short).getFieldSamples(
            u'last name'))
        assert [u'Koch'] == samples
        assert isinstance(samples[0], unicode)

    def test_Reader__getFieldSamples__5(self):
        """It returns an empty string if the only value is empty."""
        samples = list(self.getReader(self.import_file_short).getFieldSamples(
            u'firstname'))
        assert [u''] == samples
        assert isinstance(samples[0], unicode)

    def test_Reader____iter____1(self):
        """It iterates over the lines in the import file."""
        result = [[u'Koch', u'Andreas', datetime.date(1976, 1, 24)],
                  [u'Hula', u'Hanna', datetime.date(2000, 1, 1)],
                  [u'Jänsen', u'Jens', None],
                  [u'Fruma', None, datetime.date(2001, 12, 31)]]
        for index, line in enumerate(self.getReader()):
            assert result[index] == line
            for value in line:
                assert (isinstance(value, unicode) or
                        isinstance(value, datetime.date) or
                        value is None)

    def test_Reader____iter____2(self):
        """It iterates over the lines in the short import file."""
        result = [[u'Koch', None, datetime.date(1976, 1, 24)]]
        for index, line in enumerate(self.getReader(self.import_file_short)):
            assert result[index] == line
            for value in line:
                assert (isinstance(value, unicode) or
                        isinstance(value, datetime.date) or
                        value is None)



import zope.interface.verify
import icemac.ab.importer.interfaces
import icemac.ab.importer.importer


def test_importer__Importer__1():
    """It fulfills the `IImporter` interface."""
    assert zope.interface.verify.verifyObject(
        icemac.ab.importer.interfaces.IImporter,
        icemac.ab.importer.importer.Importer())



# -*- coding: utf-8 -*-
import zope.interface
import zope.schema


class IImporter(zope.interface.Interface):
    """Importer and storage for import files."""


class IImportFileContainer(zope.interface.Interface):
    """Marker interface for container which can contain files."""


class IImportFile(zope.interface.Interface):
    """Marker interface for import files."""


class IImportFileReader(zope.interface.Interface):
    """Reader for an import file."""

    title = zope.schema.TextLine(
        title=u"User understandable name of the reader.", readonly=True)

    def canRead(file_handle):
        """Tell whether this reader is able to read the given file.

        A reader is able to read a file when:

          - it can open it successfully and

          - `getFieldNames` returns at least one field.

        When an exception occorures, the reader is not able to read the file,
        too.

        This method is a class method!

        """

    def open(file_handle):
        """Create a new file reader and use `file_handle` to read from.

        This method is a class method!

        """

    def getFieldNames():
        """Get an iterable of the names of the fields in the file.

        The names are unicode strings.

        """

    def getFieldSamples(field_name):
        """Get an iterable of up to three sample values for a field.

        The values are unicode strings.
        Date values are represented as ISO-date resp. empty string.

        """

    def __iter__():
        """Iterate over the file.

        Yields a list for each data row in the file. The values are unicode
        strings or `None`.  Date values are `datetime.date` objects or
        `None`.

        The indexes are the indexes of the corresponding field names in
        `getFieldNames`.

        """



def test_wizard__end_to_end__1(address_book, browser, ImportFileFactory):
    """It allows to import data from a file into the address book."""
    ImportFileFactory(address_book, u'first.csv', [
        "firstname,last_name,addr_city",
        "Rene Manfred,Kohn,Hiernase",
        "Jurgen,Krafft,Dortmund"])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    # The first step is to choose a reader for the import file. Only the
    # readers capable to import the file are displayed:

    assert (['CSV file (comma separated fields, ISO-dates, UTF-8 encoded)'] ==
            browser.getControl('Import file format').displayOptions)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    # For each field in the address book the corresponding field in the import
    # file can be chosen. Not only the names of the fields in the import file
    # are shown but also some sample values:
    assert [
        'No value',
        'firstname (Rene Manfred, Jurgen)',
        'last_name (Kohn, Krafft)',
        'addr_city (Hiernase, Dortmund)'
    ] == browser.getControl('first name').displayOptions
    browser.getControl('first name').displayValue = ['firstname']
    browser.getControl('last name').displayValue = ['last_name']
    browser.getControl('city').displayValue = ['addr_city']
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    # In the next step the imported data is displayed as it was stored in the
    # address book.
    assert [
        'Rene Manfred',
        'Kohn',
        'Hiernase',
        'DE',
        'Jurgen',
        'Krafft',
        'Dortmund',
        'DE'] == browser.etree.xpath('//table/tbody/tr/td/text()')
    # The user can decide to keep the values:
    assert ['yes'] == browser.getControl('Keep imported data?').displayValue
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_COMPLETE_URL == browser.url
    # The last step tells that the import is complete. The `complete` button
    # leads back to the list of import files:
    assert 'Import complete.' in browser.contents
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    # The import file is still displayed:
    assert ['first.csv',
            'Delete', 'Import'] == browser.etree.xpath('//td/a/text()')
    # The imported persons are displayed on the person list:
    browser.open(browser.PERSONS_LIST_URL)
    assert ['Kohn', 'Rene Manfred',
            'Krafft', 'Jurgen'] == browser.etree.xpath('//td/a/text()')



# -*- coding: utf-8 -*-
from icemac.addressbook.i18n import _
import icemac.ab.importer.browser.wizard.base
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import icemac.addressbook.sources
import icemac.truncatetext
import xml.sax.saxutils
import z3c.table.column
import zope.i18n
import zope.interface
import zope.schema
import zope.security.proxy


class ContainerColumn(z3c.table.column.GetAttrColumn):
    """Context is a container, search values in container items."""
    # Search only objects providing this interface:
    interface = None
    # When value provides this interface, lookup atribute on container:
    container_interface = None
    # Index in the list of found objects:
    index = 0
    # Attr to look up:
    attrName = None
    # Interface of the attribute (needed to look up user defined fields):
    attrInterface = None

    def getObject(self, container):
        if (self.container_interface and
                self.container_interface == self.interface):
            return container
        candidates = list(icemac.addressbook.utils.iter_by_interface(
            container, self.interface))
        if self.index < len(candidates):
            return candidates[self.index]
        return None

    def getRawValue(self, item):
        # user fields are stored in annotation which are not
        # accessible with security proxy. This is no security hole
        # here as only administators may access the importer.
        obj = zope.security.proxy.getObject(self.getObject(item))
        if obj is not None:
            obj = self.attrInterface(obj)
        return self.getValue(obj)

    def getRenderedValue(self, item):
        value = self.getRawValue(item)
        if value is None:
            value = u''
        return value

    def renderCell(self, item):
        return self.getRenderedValue(item)


class CountryColumn(ContainerColumn):
    """ContainerColumn for gocept.country objects."""

    def getRawValue(self, obj):
        country = super(CountryColumn, self).getRawValue(obj)
        if country:
            country = country.token
        return country


class TruncatedContentColumn(ContainerColumn):
    """Truncate content to `lenght` characters."""
    # This class does not use
    # icemac.addressbook.browser.table.TruncatedContentColumn as base
    # class, as the value is computed completely differently.

    length = 20  # number of characters to display
    ellipsis = u'…'  # ellipsis sign

    def getRawValue(self, obj):
        value = super(TruncatedContentColumn, self).getRawValue(obj)
        return icemac.truncatetext.truncate(value, self.length, self.ellipsis)


def columnFactory(column):
    """Create a factory which returns the column, so the column can be used as
    an adapter without instanciating it."""
    def factory(context, field):
        return column
    return factory


ContainerColumnFactory = columnFactory(ContainerColumn)
CountryColumnFactory = columnFactory(CountryColumn)
TruncatedContentColumnFactory = columnFactory(TruncatedContentColumn)
KeywordsColumnFactory = columnFactory(
    icemac.addressbook.browser.table.KeywordsColumn)


class ReviewFields(zope.interface.Interface):
    """Fields for review form."""

    keep = zope.schema.Choice(title=_(u'Keep imported data?'),
                              source=icemac.addressbook.sources.yes_no_source)


class ImportedTable(icemac.addressbook.browser.table.Table):
    """Table displaying imported data."""

    sortOn = None
    no_rows_message = _(u'There was nothing to import in the import file.')

    def setUpColumns(self):
        cols = []
        weight = 0
        entities = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        for entity in entities.getMainEntities():
            if entity.class_name == 'icemac.addressbook.person.Person':
                entries_number = 1
                main_prefix = ''
            else:
                entries_number = self.session['entries_number']
                main_prefix = _(u'main')
            for index in xrange(entries_number):
                first = True
                for field_name, field in entity.getFields():
                    weight += 1
                    if first:
                        if index == 0:
                            title_prefix = main_prefix
                        else:
                            title_prefix = _(u'other')
                        title = _('${prefix} ${title}',
                                  mapping=dict(prefix=title_prefix,
                                               title=entity.title))
                        header = _(
                            '<i>${prefix}</i><br />${title}',
                            mapping=dict(prefix=title, title=field.title))
                        first = False
                    else:
                        header = _('<br />${title}',
                                   mapping=dict(title=field.title))
                    cols.append(self._create_col(
                        entity, field, field_name, weight, header, index))
        return cols

    def renderRow(self, row, cssClass=None):
        rendered_row = super(ImportedTable, self).renderRow(row, cssClass)
        return u'\n'.join((rendered_row,
                           self._renderErrors(row[0][0], cssClass)))

    def _create_col(self, entity, field, field_name, weight, header, index):
        """Create a single column for the field."""
        # try named adapter first
        column = zope.component.queryMultiAdapter(
            (self.context, field), z3c.table.interfaces.IColumn,
            name=field_name)
        if column is None:
            # the default adapter needs to exist
            column = zope.component.getMultiAdapter(
                (self.context, field), z3c.table.interfaces.IColumn)

        return z3c.table.column.addColumn(
            self, column, field_name, weight=weight, header=header,
            container_interface=icemac.addressbook.interfaces.IPerson,
            interface=entity.interface, attrInterface=field.interface,
            attrName=field_name, index=index)

    def _renderErrors(self, item, cssClass):
        errors = self.session['import_errors'][item.__name__]
        result = []
        if errors:
            result.extend([u'<tr class="%s">' % cssClass,
                           u'<td colspan="%s">' % len(self.columns),
                           zope.i18n.translate(_(u'Errors:'),
                                               context=self.request),
                           u'<ul class="errors">'])
            for error in errors:
                result.append(
                    u'<li>%s</li>' % xml.sax.saxutils.escape(unicode(
                        zope.i18n.translate(error, context=self.request))))
            result.extend([u'</ul>',
                           u'</td>',
                           u'</tr>'])
        return u'\n'.join(result)

    @property
    def session(self):
        return icemac.ab.importer.browser.wizard.base.get_file_session(
            self.context, self.request)

    @property
    def values(self):
        addressbook = icemac.addressbook.interfaces.IAddressBook(self.context)
        for id in self.session.get('imported', []):
            yield addressbook[id]


class Review(icemac.ab.importer.browser.wizard.base.FileSessionStorageStep):

    interface = ReviewFields
    title = label = _(u'Review imported data')

    def renderImportedTable(self):
        table = ImportedTable(self.context, self.request)
        table.update()
        return table.render()

    def render(self):
        result = super(Review, self).render()
        if self.found_errors:
            # make sure data containing errors is not stored
            icemac.ab.importer.browser.wizard.base.delete_imported_data(self)
        return result

    def applyChanges(self, data):
        super(Review, self).applyChanges(data)
        if not data['keep']:
            icemac.ab.importer.browser.wizard.base.delete_imported_data(self)
        return True

    @property
    def found_errors(self):
        """Tells whether there were import errors."""
        return self.getContent().get('found_errors', False)

    @property
    def showNextButton(self):
        """Next button condition."""
        return not self.found_errors



# -*- coding: utf-8 -*-
import icemac.ab.importer.reader.testing
import icemac.ab.importer.reader.csv.csv


class Test_csv__CSV(icemac.ab.importer.reader.testing.ReaderTest):
    """Testing ..csv.CSV"""

    reader_class = icemac.ab.importer.reader.csv.csv.CSV
    import_file = 'long.csv'
    import_file_short = 'short.csv'



# -*- coding: utf-8 -*-
from StringIO import StringIO
from icemac.addressbook.interfaces import IKeywords
from mock import patch
import gocept.country.db
import pytest
import zope.component


@pytest.fixture('function')
def example_data():
    """Construct an import file which contains errors for these constraints:

     - last name of the person must not be empty

      - birth date column must be empty or contain a valid date

      - e-mail address must be valid

      - home page address must be a URI

      - country must be either the ISO country code or the country name

      - data line must contain enough fields

    Not each field must be selected when mapping the fields. (The Field `asdf`
    is used as an example here.)

    """
    file_data = StringIO()
    file_data.write('\n'.join([
        'last_name,birth_date,e_mail_addr,www,asdf,countries,pnotes',
        ',1999-09-09,,,asdf,,',  # missing last name
        '"wrong date",1981,,,,,,',
        '"wrong e-mail",,i@me@de,,,,',
        '"wrong home page address",,i@me.de,asdf,,,',
        '"wrong country",,,,,D,',
        '"country ISO code",,,,,AT,',
        '"not enough fields",,',
        '" nice "," 2006-06-06", r6@ab.info," http://www.r6.sf.net",,'
        '" Switzerland ",'
        '"  my r e a l l y   long, but also really nice  notes    "']))
    file_data.seek(0)
    return file_data


def test_review__ImportedTable__renderRow__1(
        address_book, browser, example_data):
    """It renders error messages if data does not match the constraints.

    There are a number of constraints the imported data must fulfill
    otherwise error messages are displayed. Most constraints are tested here
    together.

    White spaces at beginning and end are striped. When a notes field
    contains more then 20 characters it get nicely truncated. (See line
    with `last_name` ``nice`` for both cases.)

    """
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_ADD_URL)
    # Upload the example file:
    browser.getControl('file').add_file(
        example_data, 'text/plain', 'errors.csv')
    browser.getControl('Add').click()
    assert '"errors.csv" added.' == browser.message
    browser.getLink('Import', index=1).click()
    assert browser.IMPORTER_FILE_IMPORT_URL == browser.url
    # Map the fields:
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = [
        'last_name (wrong date, wrong e-mail)']
    browser.getControl('birth date').displayValue = [
        'birth_date (1999-09-09, 1981)']
    browser.getControl('notes', index=0).displayValue = ['pnotes']
    browser.getControl('e-mail address').displayValue = [
        'e_mail_addr (i@me@de)']
    browser.getControl('country').displayValue = ['countries']
    browser.getControl('URL').displayValue = ['www']
    with patch('gocept.country.CountrySource.getValues') as getValues:
        getValues.return_value = [
            gocept.country.db.Country('AF'),
            gocept.country.db.Country('AT'),
            gocept.country.db.Country('CH'),
            gocept.country.db.Country('DE'),
        ]
        browser.getControl('Next').click()
    # The review step shows the error messages:
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    assert [
        'person -- last name: Required input is missing.',
        'person -- birth date: 1981 is no valid date.',
        'e-mail address -- e-mail address: i@me@de is not a valid e-mail '
        'address.',
        'home page address -- URL: The specified URI is not valid.',
        'postal address -- country: Value D is not allowed. Allowed values '
        'are: AF, AT, CH, DE',
        'home page address: Not enough data fields in row.',
        'person: Not enough data fields in row.',
        'postal address: Not enough data fields in row.',
    ] == browser.etree.xpath('//ul[@class="errors"]/li/text()')
    # As there are errors, there is no `next` button:
    assert ['form.buttons.back'] == browser.submit_control_names
    # Selecting the `back` button shows that the selected mapping has been
    # kept:
    browser.getControl('Back').click()
    assert (['last_name (wrong date, wrong e-mail)'] ==
            browser.getControl('last name').displayValue)
    assert (['birth_date (1999-09-09, 1981)'] ==
            browser.getControl('birth date').displayValue)
    assert (['pnotes'] == browser.getControl('notes', index=0).displayValue)
    assert (['e_mail_addr (i@me@de)'] ==
            browser.getControl('e-mail address').displayValue)
    assert (['countries'] == browser.getControl('country').displayValue)
    assert (['www'] == browser.getControl('URL').displayValue)
    browser.getControl('Next').click()
    # As the user can't go further from the review step he could upload another
    # import file or abort the import right now by selecting any link outside
    # the importer. None of the data rows got imported as there were errors:
    browser.open(browser.PERSONS_LIST_URL)
    assert 'There are no persons entered yet' in browser.contents


def test_review__ImportedTable__renderRow__2(
        address_book, browser, some_user_defined_fields, ImportFileFactory):
    """It renders errors if content for user defined fields is invalid."""
    ImportFileFactory(address_book, u'fail.csv', [
        'last_name,foto,meet,states,numbers,costs,mailbox',
        'wrong bool,2009-10-31,,,,,',
        'wrong bool 2,asdf,,,,,',
        'wrong datetime,,2.2.2002 13.24,,,,',
        'wrong statename,,,Berlin,,,',
        'wrong int,,,,asd,,',
        'wrong decimal,,,,,qwe,'])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = [
        'last_name (wrong bool, wrong bool 2, wrong datetime)']
    browser.getControl('photo permission?').displayValue = [
        'foto (2009-10-31, asdf)']
    browser.getControl('last seen').displayValue = ['meet (2.2.2002 13.24)']
    browser.getControl('state').displayValue = ['states']
    browser.getControl('number of letters').displayValue = ['numbers']
    browser.getControl('cost per minute').displayValue = ['costs']
    browser.getControl('mail box text').displayValue = ['mailbox']
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    assert [
        'person -- photo permission?: Value 2009-10-31 is not allowed. '
        'Allowed values are: yes, true, no, false',
        'person -- photo permission?: Value asdf is not allowed. '
        'Allowed values are: yes, true, no, false',
        'person -- last seen: 2.2.2002 13.24 is no valid datetime. '
        'Must match to format string "%Y-%m-%d %H:%M".',
        'postal address -- state: Value Berlin is not allowed. '
        'Allowed values are: Sachsen, Sachsen-Anhalt, Brandenburg',
        'postal address -- number of letters: asd is not a valid integer '
        'number.',
        'phone number -- cost per minute: qwe is not a valid decimal number.',
    ] == browser.etree.xpath('//ul[@class="errors"]/li/text()')


def test_review__ImportedTable__renderTable__1(
        address_book, browser, ImportFileFactory):
    """It renders an error message if there is nothing to import."""
    ImportFileFactory(address_book, u'header.csv', ['last_name'])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = ['last_name']
    browser.getControl('Next').click()
    # On the review step a message tells that there is nothing to import:
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    assert 'There was nothing to import in the import file' in browser.contents
    # It is possible to go to the complete step as there are no errors:
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_COMPLETE_URL == browser.url
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url


def test_review__Review__applyChanges__1(
        address_book, browser, ImportFileFactory, KeywordFactory):
    """It allows to discard the imported data.

    Newly created keywords get also deleted, but existing ones are kept.
    """
    KeywordFactory(address_book, u'family')
    ImportFileFactory(address_book, u'abc.csv', [
        'last_name,xkeywords',
        'Abc,"ccc, company"'])
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_IMPORT_URL)
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_MAP_URL == browser.url
    browser.getControl('last name').displayValue = ['last_name (Abc)']
    browser.getControl('keywords').displayValue = ['xkeywords (ccc, company)']
    browser.getControl('Next').click()
    # The review step shows the result and allows to discard the imported data:
    assert browser.IMPORTER_IMPORT_REVIEW_URL == browser.url
    assert (['Abc', 'ccc, company', 'DE'] ==
            browser.etree.xpath('//table/tbody/tr/td/text()'))
    browser.getControl('Keep imported data?').displayValue = ['no']
    # Completing the import deletes the imported data and keywords:
    browser.getControl('Next').click()
    assert browser.IMPORTER_IMPORT_COMPLETE_URL == browser.url
    browser.getControl('Complete').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    # No new keywords have been created:
    keywords = zope.component.getUtility(IKeywords, context=address_book)
    assert [u'family'] == sorted(x.title for x in keywords.get_keywords())
    # No new persons have been created:
    assert [] == list(address_book)

# delete ../keywords.txt



import icemac.ab.importer.interfaces
import icemac.addressbook.browser.menus.menu


importer_views = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    icemac.ab.importer.interfaces.IImporter,
    icemac.ab.importer.interfaces.IImportFile)



"""Database initialisation and upgrading."""

import zope.generations.generations


GENERATION = 0


manager = zope.generations.generations.SchemaManager(
    minimum_generation=GENERATION,
    generation=GENERATION,
    package_name='icemac.ab.importer.generations')



from icemac.addressbook.addressbook import AddressBookCreated
from icemac.ab.importer.install import install_importer
from icemac.ab.importer.interfaces import IImporter


def test_install__install_importer__1(assert_address_book):
    """It creates an importer attribute."""
    install_importer(AddressBookCreated(assert_address_book.address_book))
    assert_address_book.has_attribute('importer', IImporter)


def test_install__install_importer__2(assert_address_book):
    """It does not break if it gets called twice."""
    event = AddressBookCreated(assert_address_book.address_book)
    install_importer(event)
    install_importer(event)
    assert_address_book.has_attribute('importer', IImporter)



from icemac.ab.importer.interfaces import IImportFile
import pytest

# Fixtures


@pytest.fixture('function')
def importer_link(browser):
    """Generate the importer link rendered on the master data view."""
    return '<a href="{}"><span>Import data</span></a>'.format(
        browser.IMPORTER_OVERVIEW_URL)


# Tests


def test_importer__CRUD__1(address_book, browser, import_file):
    """It allows to upload, edit, list and delete files for import."""
    browser.login('mgr')
    browser.open(browser.MASTER_DATA_URL)
    # Overview is empty by default
    browser.getLink('Import data').click()
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    assert "No import files uploaded, yet" in browser.contents
    # Create a file
    browser.getLink('file').click()
    assert browser.IMPORTER_FILE_ADD_URL == browser.url
    browser.getControl('file').add_file(
        import_file('Import data file'), 'text/plain', 'file.txt')
    browser.getControl('Add').click()
    assert '"file.txt" added.' == browser.message
    # List the file
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    assert '>file.txt</a>' in browser.contents
    assert browser.IMPORTER_FILE_IMPORT_URL == browser.getLink(
        'Import', index=1).url
    # Read the uploaded file
    browser.getLink('file.txt').click()
    assert browser.IMPORTER_FILE_EDIT_URL == browser.url
    assert browser.getControl('name').value == 'file.txt'
    assert 'text/plain' == browser.getControl('Mime Type').value
    browser.getLink('Download file').click()
    assert browser.IMPORTER_FILE_DOWNLOAD_URL == browser.url
    assert 'text/plain' == browser.headers['content-type']
    assert ('attachment; filename=file.txt' ==
            browser.headers['content-disposition'])
    assert 'Import data file' == browser.contents
    # It is possible to upload a new file instead of the previously uploaded
    # one:
    browser.open(browser.IMPORTER_FILE_EDIT_URL)
    browser.getControl('file', index=1).add_file(
        import_file('Import2 data2 file2'), 'text/csv', 'file2.csv')
    browser.getControl('Save').click()
    assert 'Data successfully updated.' == browser.message
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    # Delete file
    browser.getLink('Delete').click()
    assert browser.IMPORTER_FILE_DELETE_URL == browser.url
    browser.getControl('Yes').click()
    assert '"file2.csv" deleted.' == browser.message
    assert browser.IMPORTER_OVERVIEW_URL == browser.url
    # Get rid of flash message containing `file2.csv`:
    browser.open(browser.IMPORTER_OVERVIEW_URL)
    assert 'file2.csv' not in browser.contents


def test_importer__Overview__1(address_book, browser, importer_link):
    """There is a link to it rendered in master data for admin."""
    browser.login('mgr')
    browser.open(browser.MASTER_DATA_URL)
    assert ([importer_link] in
            [browser.etree_to_list(x) for x in browser.etree.xpath('//li/a')])


@pytest.mark.parametrize('role', ['editor', 'visitor'])
def test_importer__Overview__2(address_book, browser, importer_link, role):
    """There is a no link to it rendered in master data for some roles."""
    browser.login(role)
    browser.open(browser.MASTER_DATA_URL)
    assert ([importer_link] not in
            [browser.etree_to_list(x) for x in browser.etree.xpath('//li/a')])


@pytest.mark.parametrize('role', ['editor', 'visitor'])
def test_importer__Overview__3(address_book, browser, role):
    """It is not allowed to be accessed by some user roles."""
    browser.login(role)
    browser.assert_forbidden(browser.IMPORTER_OVERVIEW_URL)


@pytest.mark.parametrize('role', ['editor', 'visitor'])
def test_importer__Add__1(address_book, browser, role):
    """It is not allowed to be accessed by some user roles."""
    browser.login(role)
    browser.assert_forbidden(browser.IMPORTER_FILE_ADD_URL)


def test_importer__Add__2(address_book, browser, import_file):
    """It sets the interface `IImportFile` on the created file."""
    browser.login('mgr')
    browser.open(browser.IMPORTER_FILE_ADD_URL)
    browser.getControl('file').add_file(
        import_file('Import data file'), 'text/plain', 'file.txt')
    browser.getControl('Add').click()
    assert '"file.txt" added.' == browser.message
    assert IImportFile.providedBy(address_book.importer['File'])


@pytest.mark.parametrize('role', ['editor', 'visitor'])
def test_importer__Edit__1(address_book, browser, role):
    """It is not allowed to be accessed by some user roles."""
    browser.login(role)
    browser.assert_forbidden(browser.IMPORTER_FILE_EDIT_URL)


@pytest.mark.parametrize('role', ['editor', 'visitor'])
def test_importer__Delete__1(address_book, browser, role):
    """It is not allowed to be accessed by some user roles."""
    browser.login(role)
    browser.assert_forbidden(browser.IMPORTER_FILE_DELETE_URL)



import zope.container.btree
import zope.interface
import icemac.ab.importer.interfaces


@zope.interface.implementer(icemac.ab.importer.interfaces.IImporter)
class Importer(zope.container.btree.BTreeContainer):
    """Importer containing files for import."""



from icemac.addressbook.interfaces import IPerson
from icemac.addressbook.interfaces import IPhoneNumber, IPostalAddress
import pytest


@pytest.fixture('function')
def some_user_defined_fields(address_book, FieldFactory):
    """Create some user defined fields."""
    FieldFactory(address_book, IPerson, u'Bool', u'photo permission?')
    FieldFactory(address_book, IPerson, u'Datetime', u'last seen')
    FieldFactory(address_book, IPostalAddress, u'Choice', u'state',
                 values=[u'Sachsen', u'Sachsen-Anhalt', u'Brandenburg'])
    FieldFactory(address_book, IPostalAddress, u'Int', u'number of letters')
    FieldFactory(address_book, IPhoneNumber, u'Decimal', u'cost per minute')
    FieldFactory(address_book, IPhoneNumber, u'Text', u'mail box text')
